; trigger.asm, remote shutter release trigger for cameras
; Copyright 2010 Christopher Williams
; 
; This program is free software; you can redistribute it and/or
; modify it under the terms of the GNU General Public License
; as published by the Free Software Foundation; either version 2
; of the License, or (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

; TI-OS routines
_putc equ 0x4a2B
_puts equ 0x4a37
_clrScrn equ 0x4a82
_newline equ 0x4a5F
_homeUp equ 0x4a95
_runindicoff equ 0x4AB1
_runindicon  equ 0x4AAD

; TI-OS variables
_curRow equ 0xC00F ; cursor row
_curCol equ 0xC010 ; cursor column


; I/O ports
kbd_port equ 1
link_port equ 7

; link port equates for shutter/focus control
focus_mask equ 0x08 ; ring
shutter_mask equ 0x04 ; tip
link_mask equ 0xc0 ; default mask to OR with focus and shutter

LCDMEM equ 0xfc00

	.org 0xd748
	
	call	_runindicoff ; shut off that pesky run indicator
	
	; initialize
	call	disable_shutter
	call	disable_focus
	ld	a,0xff
	ld	(last_key_mask),a
	
	; main menu
	call	print_menu
	
; F1 - F5: out bit 6, in bits 4 3 2 1 0
; EXIT = out bit 6, in bit 6

	; main key loop
keyloop:
	halt
	ld	a,0xbf ; bit 6
	out	(kbd_port),a
	in	a,(kbd_port)
	ld	b,a ; b is current row mask
	ld	a,(last_key_mask)
	xor	b
	;ld	(LCDMEM+16),a
	ld	c,a ; c is difference
	ld	a,b
	ld	(last_key_mask),a
	
	; test each bit
	; F5
	ld	a,0x01
	and	c
	jr	z,nodiff0
	; F5 was changed
	and	b
	jr	z,down0
	; F5 is up
	call	disable_shutter
	jr	done0
down0:
	; F5 is down
	call	enable_shutter
done0:
nodiff0:
	
	; F4
	ld	a,0x02
	and	c
	jr	z,nodiff1
	; F4 was changed
	and	b
	; F4 is down
	call	z,toggle_shutter
nodiff1:
	
	
	; F2
	ld	a,0x08
	and	c
	jr	z,nodiff3
	; F2 was changed
	and	b
	; F2 is down
	call	z,toggle_focus
nodiff3:
	
	; F1
	ld	a,0x10
	and	c
	jr	z,nodiff4
	; F1 was changed
	and	b
	jr	z,down4
	; F1 is up
	call	disable_focus
	jr	done4
down4:
	; F1 is down
	call	enable_focus
done4:
nodiff4:
	
	; EXIT
	ld	a,0x40
	and	c
	jr	z,nodiff6
	; EXIT was changed
	and	b
	; EXIT is down
	jp	z,quit
nodiff6:
	
	; MORE
	ld	a,0x80
	and	c
	jr	z,nodiff7
	; MORE was changed
	and	b
	jr	nz,done7
	; MORE is down
	push	bc
	call	about
	call	print_menu
	pop	bc
done7:
nodiff7:
	
	
	jr	keyloop
	
quit:
	; reset the link output
	call	disable_shutter
	call	disable_focus
	
	call	_clrScrn
	jp	_homeUp

print_screen:
	push	hl
	call	_clrScrn
	call	_homeUp
	pop	hl
	jp	_puts

print_menu:
	ld	hl,menu_str
	jp	print_screen

menu_str:
;          "123456789012345678901"
	.if 0
	db "  F1 = Toggle Focus  "
	db "                     "
	db "F3 = Open Shutter    "
	db "F4 = Close Shutter   "
	db "F5 = Trigger Shutter "
	db "                     "
	db "MORE = About         "
	db "EXIT = Exit"
	db 0
	.endif

	.if 0
	db "   F1:Toggle Focus   "
	db "                     "
	db "------ Shutter ------"
	db " F3:Open    F4:Close "
	db "     F5:Trigger      "
	db "---------------------"
	db "MORE:About  EXIT:Exit"
	db "   [ ]         [ ]"
	db 0
	.endif

	db "------- Focus -------"
	db "F1:Trigger    F2:Hold"
	db "                     "
	db "------ Shutter ------"
	db "F4:Hold    F5:Trigger"
	db "                     "
	db "MORE:About  EXIT:Exit"
	db "    [ ]       [ ]"
	db 0

about:
	call	print_about
keyloop2:
	halt
	ld	a,0xbf ; bit 6
	out	(kbd_port),a
	in	a,(kbd_port)
	
	and	0x80
	jr	z,keyloop2
	ret

print_about:
	ld	hl,about_str
	jp	print_screen

about_str:
;          "123456789012345678901"
	db "   Remote Shutter    "
	db "   Release Trigger   "
	db "        v0.5         "
	db "                     "
	db "   Copyright 2010    "
	db "Christopher Williams "
	db "See documentation for"
	db " license information"
	db 0

; display focus and shutter status
update_status:
	ld	hl,7+256*5 ; row 7 col 5
	ld	(_curRow),hl
	ld	a,(focus)
	or	a
	ld	a,' '
	jr	z,nofocus
	ld	a,'F'
nofocus:
	call	_putc
	ld	a,15
	ld	(_curCol),a ; row 7 col 15
	ld	a,(shutter)
	or	a
	ld	a,' '
	jr	z,noshutter
	ld	a,'S'
noshutter:
	call	_putc
	ret

output_link:
	ld	de,(focus) ; d = shutter, e = focus
	ld	a,link_mask
	or	d
	or	e
	;ld	(LCDMEM),a ; debug
	out	(link_port),a
	ret
	
; trigger focus
enable_focus:
	ld	a,focus_mask
	ld	(focus),a
	call	update_status
	jp	output_link

disable_focus:
	xor	a
	ld	(focus),a
	call	update_status
	jp	output_link

toggle_focus:
	ld	hl,focus
	ld	a,(hl)
	xor	focus_mask
	ld	(hl),a
	call	update_status
	jp	output_link

; trigger the shutter
enable_shutter:
	ld	a,shutter_mask
	ld	(shutter),a
	call	update_status
	jp	output_link

; close the shutter
disable_shutter:
	xor	a
	ld	(shutter),a
	call	update_status
	jp	output_link

toggle_shutter:
	ld	hl,shutter
	ld	a,(hl)
	xor	shutter_mask
	ld	(hl),a
	call	update_status
	jp	output_link

; last keyboard mask
last_key_mask:
	db 0xff

focus db 0
shutter db 0
